@@ -1,7 +1,6 @@
 /* -*-mode:java; c-basic-offset:2; indent-tabs-mode:nil -*- */
 import com.jcraft.jsch.*;
 import java.awt.*;
-import java.awt.event.*;
 import javax.swing.*;
 
 public class UserAuthKI{
@@ -10,9 +9,15 @@ public class UserAuthKI{
     try{
       JSch jsch=new JSch();
 
-      String host=JOptionPane.showInputDialog("Enter username@hostname",
-                                              System.getProperty("user.name")+
-                                              "@localhost"); 
+      String host=null;
+      if(arg.length>0){
+        host=arg[0];
+      }
+      else{
+        host=JOptionPane.showInputDialog("Enter username@hostname",
+                                         System.getProperty("user.name")+
+                                         "@localhost"); 
+      }
       String user=host.substring(0, host.indexOf('@'));
       host=host.substring(host.indexOf('@')+1);
 
@@ -81,15 +86,14 @@ public class UserAuthKI{
                                               String instruction,
                                               String[] prompt,
                                               boolean[] echo){
-      /*
+/*
 //System.out.println("promptKeyboardInteractive");
 System.out.println("destination: "+destination);
 System.out.println("name: "+name);
 System.out.println("instruction: "+instruction);
 System.out.println("prompt.length: "+prompt.length);
 System.out.println("prompt: "+prompt[0]);
-      */
-
+*/
       panel = new JPanel();
       panel.setLayout(new GridBagLayout());
 
