@@ -1,6 +1,6 @@
 /* -*-mode:java; c-basic-offset:2; indent-tabs-mode:nil -*- */
 /*
-Copyright (c) 2002-2008 ymnk, JCraft,Inc. All rights reserved.
+Copyright (c) 2002-2009 ymnk, JCraft,Inc. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
@@ -70,7 +70,7 @@ public class ChannelForwardedTCPIP extends Channel{
                                                  ), false);
 
         daemon.setChannel(this, getInputStream(), out);
-        Object[] foo=getPort(session, rport);
+        Object[] foo=getPort(getSession(), rport);
         daemon.setArg((Object[])foo[3]);
 
         new Thread(daemon).start();
@@ -115,7 +115,7 @@ public class ChannelForwardedTCPIP extends Channel{
         buf.putInt(recipient);
         buf.putInt(i);
         buf.skip(i);
-        session.write(packet, this, i);
+        getSession().write(packet, this, i);
       }
     }
     catch(Exception e){
@@ -125,6 +125,7 @@ public class ChannelForwardedTCPIP extends Channel{
     //eof();
     disconnect();
   }
+
   void getData(Buffer buf){
     setRecipient(buf.getInt());
     setRemoteWindowSize(buf.getInt());
@@ -141,22 +142,30 @@ public class ChannelForwardedTCPIP extends Channel{
     System.err.println("orgport: "+orgport);
     */
 
+    Session _session=null;
+    try{
+      _session=getSession();
+    }
+    catch(JSchException e){
+      // session has been already down.
+    }
+
     synchronized(pool){
       for(int i=0; i<pool.size(); i++){
         Object[] foo=(Object[])(pool.elementAt(i));
-        if(foo[0]!=session) continue;
+        if(foo[0]!=_session) continue;
         if(((Integer)foo[1]).intValue()!=port) continue;
         this.rport=port;
         this.target=(String)foo[2];
         if(foo[3]==null || (foo[3] instanceof Object[])){ this.lport=-1; }
         else{ this.lport=((Integer)foo[3]).intValue(); }
-        if(foo.length>=5){
-          this.factory=((SocketFactory)foo[4]);
+        if(foo.length>=6){
+          this.factory=((SocketFactory)foo[5]);
         }
         break;
       }
       if(target==null){
-        System.err.println("??");
+        //System.err.println("??");
       }
     }
   }
@@ -190,33 +199,54 @@ public class ChannelForwardedTCPIP extends Channel{
     return bar;
   }
 
-  static void addPort(Session session, int port, String target, int lport, SocketFactory factory) throws JSchException{
+  static String normalize(String address){
+    if(address==null){ return "localhost"; }
+    else if(address.length()==0 || address.equals("*")){ return ""; }
+    else{ return address; }
+  }
+
+  static void addPort(Session session, String _address_to_bind, int port, String target, int lport, SocketFactory factory) throws JSchException{
+    String address_to_bind=normalize(_address_to_bind);
+    synchronized(pool){
+      if(getPort(session, port)!=null){
+        throw new JSchException("PortForwardingR: remote port "+port+" is already registered.");
+      }
+      Object[] foo=new Object[6];
+      foo[0]=session; foo[1]=new Integer(port);
+      foo[2]=target; foo[3]=new Integer(lport);
+      foo[4]=address_to_bind;
+      foo[5]=factory;
+      pool.addElement(foo);
+    }
+  }
+  static void addPort(Session session, String _address_to_bind, int port, String daemon, Object[] arg) throws JSchException{
+    String address_to_bind=normalize(_address_to_bind);
     synchronized(pool){
       if(getPort(session, port)!=null){
         throw new JSchException("PortForwardingR: remote port "+port+" is already registered.");
       }
       Object[] foo=new Object[5];
       foo[0]=session; foo[1]=new Integer(port);
-      foo[2]=target; foo[3]=new Integer(lport);
-      foo[4]=factory;
-      pool.addElement(foo);
-    }
-  }
-  static void addPort(Session session, int port, String daemon, Object[] arg) throws JSchException{
-    synchronized(pool){
-      if(getPort(session, port)!=null){
-        throw new JSchException("PortForwardingR: remote port "+port+" is already registered.");
-      }
-      Object[] foo=new Object[4];
-      foo[0]=session; foo[1]=new Integer(port);
       foo[2]=daemon; foo[3]=arg;
+      foo[4]=address_to_bind; 
       pool.addElement(foo);
     }
   }
   static void delPort(ChannelForwardedTCPIP c){
-    delPort(c.session, c.rport);
+    Session _session=null;
+    try{
+      _session=c.getSession();
+    }
+    catch(JSchException e){
+      // session has been already down.
+    }
+    if(_session!=null)
+      delPort(_session, c.rport);
   }
   static void delPort(Session session, int rport){
+    delPort(session, null, rport);
+  }
+  static void delPort(Session session, String address_to_bind, int rport){
     synchronized(pool){
       Object[] foo=null;
       for(int i=0; i<pool.size(); i++){
@@ -227,7 +257,13 @@ public class ChannelForwardedTCPIP extends Channel{
         break;
       }
       if(foo==null)return;
-      pool.removeElement(foo);	
+      pool.removeElement(foo);
+      if(address_to_bind==null){
+        address_to_bind=(String)foo[4];
+      }	
+      if(address_to_bind==null){
+        address_to_bind="0.0.0.0";
+      }
     }
 
     Buffer buf=new Buffer(100); // ??
@@ -243,7 +279,7 @@ public class ChannelForwardedTCPIP extends Channel{
       buf.putByte((byte) 80/*SSH_MSG_GLOBAL_REQUEST*/);
       buf.putString("cancel-tcpip-forward".getBytes());
       buf.putByte((byte)0);
-      buf.putString("0.0.0.0".getBytes());
+      buf.putString(address_to_bind.getBytes());
       buf.putInt(rport);
       session.write(packet);
     }
