@@ -1,6 +1,6 @@
 /* -*-mode:java; c-basic-offset:2; indent-tabs-mode:nil -*- */
 /*
-Copyright (c) 2002-2008 ymnk, JCraft,Inc. All rights reserved.
+Copyright (c) 2002-2009 ymnk, JCraft,Inc. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
@@ -53,41 +53,134 @@ class ChannelSession extends Channel{
     io=new IO();
   }
 
+  /**
+   * Enable the agent forwarding.
+   *
+   * @param enable
+   */
   public void setAgentForwarding(boolean enable){ 
     agent_forwarding=enable;
   }
+
+  /**
+   * Enable the X11 forwarding.
+   *
+   * @param enable
+   * @see RFC4254 6.3.1. Requesting X11 Forwarding
+   */
   public void setXForwarding(boolean enable){
     xforwading=enable; 
   }
+
+  /**
+   * @deprecated Use {@link #setEnv(String, String)} or {@link #setEnv(byte[], byte[])} instead.
+   * @see #setEnv(String, String)
+   * @see #setEnv(byte[], byte[])
+   */
   public void setEnv(Hashtable env){ 
-    this.env=env; 
+    synchronized(this){
+      this.env=env; 
+    }
   }
 
+  /**
+   * Set the environment variable. 
+   * If <code>name</code> and <code>value</code> are needed to be passed 
+   * to the remote in your faivorite encoding,use 
+   * {@link #setEnv(byte[], byte[])}.
+   *
+   * @param name A name for environment variable.
+   * @param value A value for environment variable.
+   * @see RFC4254 6.4 Environment Variable Passing
+   */
+  public void setEnv(String name, String value){
+    setEnv(name.getBytes(), value.getBytes());
+  }
+
+  /**
+   * Set the environment variable.
+   *
+   * @param name A name of environment variable.
+   * @param value A value of environment variable.
+   * @see #setEnv(String, String)
+   * @see RFC4254 6.4 Environment Variable Passing
+   */
+  public void setEnv(byte[] name, byte[] value){
+    synchronized(this){
+      getEnv().put(name, value);
+    }
+  }
+
+  private Hashtable getEnv(){
+    if(env==null)
+      env=new Hashtable();
+    return env;
+  }
+
+  /**
+   * Allocate a Pseudo-Terminal.
+   *
+   * @param enable
+   * @see RFC4254 6.2. Requesting a Pseudo-Terminal
+   */
   public void setPty(boolean enable){ 
     pty=enable; 
   }
 
+  /**
+   * Set the terminal mode.
+   * 
+   * @param terminal_mode
+   */
   public void setTerminalMode(byte[] terminal_mode){
     this.terminal_mode=terminal_mode;
   }
+
+  /**
+   * Change the window dimension interactively.
+   * 
+   * @param col terminal width, columns
+   * @param row terminal height, rows
+   * @param wp terminal width, pixels
+   * @param hp terminal height, pixels
+   * @see RFC4254 6.7. Window Dimension Change Message
+   */
   public void setPtySize(int col, int row, int wp, int hp){
-    if(!pty){
+    setPtyType(this.ttype, col, row, wp, hp);
+    if(!pty || !isConnected()){
       return;
     }
     try{
       RequestWindowChange request=new RequestWindowChange();
       request.setSize(col, row, wp, hp);
-      request.request(session, this);
+      request.request(getSession(), this);
     }
     catch(Exception e){
       //System.err.println("ChannelSessio.setPtySize: "+e);
     }
   }
 
+  /**
+   * Set the terminal type.
+   * This method is not effective after Channel#connect().
+   *
+   * @param ttype terminal type(for example, "vt100")
+   * @see #setPtyType(String, int, int, int, int)
+   */
   public void setPtyType(String ttype){
     setPtyType(ttype, 80, 24, 640, 480);
   }
 
+  /**
+   * Set the terminal type.
+   * This method is not effective after Channel#connect().
+   *
+   * @param ttype terminal type(for example, "vt100")
+   * @param col terminal width, columns
+   * @param row terminal height, rows
+   * @param wp terminal width, pixels
+   * @param hp terminal height, pixels
+   */
   public void setPtyType(String ttype, int col, int row, int wp, int hp){
     this.ttype=ttype;
     this.tcol=col;
@@ -97,15 +190,16 @@ class ChannelSession extends Channel{
   }
 
   protected void sendRequests() throws Exception{
+    Session _session=getSession();
     Request request;
     if(agent_forwarding){
       request=new RequestAgentForwarding();
-      request.request(session, this);
+      request.request(_session, this);
     }
 
     if(xforwading){
       request=new RequestX11();
-      request.request(session, this);
+      request.request(_session, this);
     }
 
     if(pty){
@@ -115,26 +209,30 @@ class ChannelSession extends Channel{
       if(terminal_mode!=null){
         ((RequestPtyReq)request).setTerminalMode(terminal_mode);
       }
-      request.request(session, this);
+      request.request(_session, this);
     }
 
     if(env!=null){
-      for(Enumeration _env=env.keys() ; _env.hasMoreElements() ;) {
-        String name=(String)(_env.nextElement());
-        String value=(String)(env.get(name));
+      for(Enumeration _env=env.keys(); _env.hasMoreElements();){
+        Object name=_env.nextElement();
+        Object value=env.get(name);
         request=new RequestEnv();
-        ((RequestEnv)request).setEnv(name, value);
-        request.request(session, this);
+        ((RequestEnv)request).setEnv(toByteArray(name), 
+                                     toByteArray(value));
+        request.request(_session, this);
       }
     }
   }
 
+  private byte[] toByteArray(Object o){
+    if(o instanceof String){
+      return ((String)o).getBytes();
+    }
+    return (byte[])o;
+  }
+
   public void run(){
     //System.err.println(this+":run >");
-/*
-    if(thread!=null){ return; }
-    thread=Thread.currentThread();
-*/
 
     Buffer buf=new Buffer(rmpsize);
     Packet packet=new Packet(buf);
@@ -161,7 +259,7 @@ class ChannelSession extends Channel{
         buf.putInt(recipient);
         buf.putInt(i);
         buf.skip(i);
-	session.write(packet, this, i);
+	getSession().write(packet, this, i);
       }
     }
     catch(Exception e){
