@@ -1,6 +1,6 @@
 /* -*-mode:java; c-basic-offset:2; indent-tabs-mode:nil -*- */
 /*
-Copyright (c) 2004,2005,2006 ymnk, JCraft,Inc. All rights reserved.
+Copyright (c) 2004-2009 ymnk, JCraft,Inc. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
@@ -35,7 +35,7 @@ public interface HostKeyRepository{
   final int CHANGED=2;
 
   int check(String host, byte[] key);
-  void add(String host, byte[] key, UserInfo ui);
+  void add(HostKey hostkey, UserInfo ui);
   void remove(String host, String type);
   void remove(String host, String type, byte[] key);
   String getKnownHostsRepositoryID();
