@@ -1,6 +1,6 @@
 /* -*-mode:java; c-basic-offset:2; indent-tabs-mode:nil -*- */
 /*
-Copyright (c) 2002,2003,2004,2005,2006 ymnk, JCraft,Inc. All rights reserved.
+Copyright (c) 2002-2009 ymnk, JCraft,Inc. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
@@ -35,5 +35,5 @@ public interface MAC{
   void init(byte[] key) throws Exception; 
   void update(byte[] foo, int start, int len);
   void update(int foo);
-  byte[] doFinal();
+  void doFinal(byte[] buf, int offset);
 }
