@@ -1,6 +1,6 @@
 /* -*-mode:java; c-basic-offset:2; indent-tabs-mode:nil -*- */
 /*
-Copyright (c) 2002-2008 ymnk, JCraft,Inc. All rights reserved.
+Copyright (c) 2002-2009 ymnk, JCraft,Inc. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
@@ -33,7 +33,7 @@ import java.io.*;
 import java.net.*;
 
 public class Session implements Runnable{
-  static private final String version="JSCH-0.1.37";
+  static private final String version="JSCH-0.1.42";
 
   // http://ietf.org/internet-drafts/draft-ietf-secsh-assignednumbers-01.txt
   static final int SSH_MSG_DISCONNECT=                      1;
@@ -105,6 +105,7 @@ public class Session implements Runnable{
   private boolean isAuthed=false;
 
   private Thread connectThread=null;
+  private Object lock=new Object();
 
   boolean x11_forwarding=false;
   boolean agent_forwarding=false;
@@ -237,14 +238,16 @@ public class Session implements Runnable{
 
         if(buf.buffer[i-1]==10){    // 0x0a
           i--;
-          if(buf.buffer[i-1]==13){  // 0x0d
+          if(i>0 && buf.buffer[i-1]==13){  // 0x0d
             i--;
           }
         }
 
-        if(i>4 && (i!=buf.buffer.length) &&
-           (buf.buffer[0]!='S'||buf.buffer[1]!='S'||
-            buf.buffer[2]!='H'||buf.buffer[3]!='-')){
+        if(i<=3 || 
+           ((i!=buf.buffer.length) &&
+            (buf.buffer[0]!='S'||buf.buffer[1]!='S'||
+             buf.buffer[2]!='H'||buf.buffer[3]!='-'))){
+          // It must not start with 'SSH-'
           //System.err.println(new String(buf.buffer, 0, i);
           continue;
         }
@@ -454,12 +457,21 @@ public class Session implements Runnable{
       }
 
       isAuthed=true;
-      connectThread=new Thread(this);
-      connectThread.setName("Connect thread "+host+" session");
-      if(daemon_thread){
-        connectThread.setDaemon(daemon_thread);
+
+      synchronized(lock){
+        if(isConnected){
+          connectThread=new Thread(this);
+          connectThread.setName("Connect thread "+host+" session");
+          if(daemon_thread){
+            connectThread.setDaemon(daemon_thread);
+          }
+          connectThread.start();
+        }
+        else{
+          // The session has been already down and
+          // we don't have to start new thread.
+        }
       }
-      connectThread.start();
     }
     catch(Exception e) {
       in_kex=false;
@@ -497,7 +509,11 @@ public class Session implements Runnable{
     else{
       I_S=new byte[j-1-buf.getByte()];
     }
-    System.arraycopy(buf.buffer, buf.s, I_S, 0, I_S.length);
+   System.arraycopy(buf.buffer, buf.s, I_S, 0, I_S.length);
+
+   if(!in_kex){     // We are in rekeying activated by the remote!
+     send_kexinit();
+   }
 
     guess=KeyExchange.guess(I_S, I_C);
     if(guess==null){
@@ -557,6 +573,8 @@ public class Session implements Runnable{
     // string    compression_algorithms_server_to_client
     // string    languages_client_to_server
     // string    languages_server_to_client
+    Buffer buf = new Buffer();                // send_kexinit may be invoked
+    Packet packet = new Packet(buf);          // by user thread.
     packet.reset();
     buf.putByte((byte) SSH_MSG_KEXINIT);
     synchronized(random){
@@ -812,6 +830,22 @@ key_type+" key fingerprint is "+key_fprint+".\n"+
         System.arraycopy(buf.buffer, 0, foo, 0, buf.index);
         buf.buffer=foo;
       }
+
+      if((j%s2ccipher_size)!=0){
+        String message="Bad packet length "+j;
+        if(JSch.getLogger().isEnabled(Logger.FATAL)){
+          JSch.getLogger().log(Logger.FATAL, message); 
+        }
+        packet.reset();
+	buf.putByte((byte)SSH_MSG_DISCONNECT);
+	buf.putInt(3);
+	buf.putString(message.getBytes());
+	buf.putString("en".getBytes());
+	write(packet);
+	disconnect();
+	throw new JSchException("SSH_MSG_DISCONNECT: "+message);
+      }
+
       if(j>0){
 	io.getByte(buf.buffer, buf.index, j); buf.index+=(j);
 	if(s2ccipher!=null){
@@ -855,8 +889,8 @@ key_type+" key fingerprint is "+key_fprint+".\n"+
 	int reason_code=buf.getInt();
 	byte[] description=buf.getString();
 	byte[] language_tag=buf.getString();
-	throw new JSchException("SSH_MSG_DISCONNECT:"+
-				" "+reason_code+
+	throw new JSchException("SSH_MSG_DISCONNECT: "+
+				    reason_code+
 				" "+new String(description)+
 				" "+new String(language_tag));
 	//break;
@@ -1129,12 +1163,15 @@ key_type+" key fingerprint is "+key_fprint+".\n"+
     }
     _write(packet);
   }
-  private synchronized void _write(Packet packet) throws Exception{
-     encode(packet);
-     if(io!=null){
-       io.put(packet);
-       seqo++;
-     }
+
+  private void _write(Packet packet) throws Exception{
+    synchronized(lock){
+      encode(packet);
+      if(io!=null){
+        io.put(packet);
+        seqo++;
+      }
+    }
   }
 
   Runnable thread;
@@ -1481,10 +1518,12 @@ break;
     PortWatcher.delPort(this);
     ChannelForwardedTCPIP.delPort(this);
 
-    synchronized(connectThread){
-      Thread.yield();
-      connectThread.interrupt();
-      connectThread=null;
+    synchronized(lock){
+      if(connectThread!=null){
+        Thread.yield();
+        connectThread.interrupt();
+        connectThread=null;
+      }
     }
     thread=null;
     try{
@@ -1554,7 +1593,7 @@ break;
     setPortForwardingR(null, rport, host, lport, sf);
   }
   public void setPortForwardingR(String bind_address, int rport, String host, int lport, SocketFactory sf) throws JSchException{
-    ChannelForwardedTCPIP.addPort(this, rport, host, lport, sf);
+    ChannelForwardedTCPIP.addPort(this, bind_address, rport, host, lport, sf);
     setPortForwarding(bind_address, rport);
   }
 
@@ -1565,7 +1604,7 @@ break;
     setPortForwardingR(null, rport, daemon, arg);
   }
   public void setPortForwardingR(String bind_address, int rport, String daemon, Object[] arg) throws JSchException{
-    ChannelForwardedTCPIP.addPort(this, rport, daemon, arg);
+    ChannelForwardedTCPIP.addPort(this, bind_address, rport, daemon, arg);
     setPortForwarding(bind_address, rport);
   }
 
@@ -1586,17 +1625,7 @@ break;
     Buffer buf=new Buffer(100); // ??
     Packet packet=new Packet(buf);
 
-    String address_to_bind="localhost";
-    if(bind_address==null){
-      //address_to_bind="localhost";
-    }
-    else if(bind_address.length()==0 ||
-            bind_address.equals("*")){
-      address_to_bind="";
-    }
-    else{
-      address_to_bind=bind_address;
-    }
+    String address_to_bind=ChannelForwardedTCPIP.normalize(bind_address);
 
     try{
       // byte SSH_MSG_GLOBAL_REQUEST 80
@@ -1610,7 +1639,6 @@ break;
 //      buf.putByte((byte)0);
       buf.putByte((byte)1);
       buf.putString(address_to_bind.getBytes());
-      //buf.putString("0.0.0.0".getBytes());
       buf.putInt(rport);
       write(packet);
     }
@@ -1682,10 +1710,9 @@ break;
   }
 
   void addChannel(Channel channel){
-    channel.session=this;
+    channel.setSession(this);
   }
 
-//  public Channel getChannel(){ return channel; }
   public void setProxy(Proxy proxy){ this.proxy=proxy; }
   public void setHost(String host){ this.host=host; }
   public void setPort(int port){ this.port=port; }
@@ -1712,20 +1739,24 @@ break;
     setConfig((java.util.Hashtable)newconf);
   }
  
-  public synchronized void setConfig(java.util.Hashtable newconf){
-    if(config==null) 
-      config=new java.util.Hashtable();
-    for(java.util.Enumeration e=newconf.keys() ; e.hasMoreElements() ;) {
-      String key=(String)(e.nextElement());
-      config.put(key, (String)(newconf.get(key)));
+  public void setConfig(java.util.Hashtable newconf){
+    synchronized(lock){
+      if(config==null) 
+        config=new java.util.Hashtable();
+      for(java.util.Enumeration e=newconf.keys() ; e.hasMoreElements() ;) {
+        String key=(String)(e.nextElement());
+        config.put(key, (String)(newconf.get(key)));
+      }
     }
   }
 
-  public synchronized void setConfig(String key, String value){
-    if(config==null){
-      config=new java.util.Hashtable();
+  public void setConfig(String key, String value){
+    synchronized(lock){ 
+      if(config==null){
+        config=new java.util.Hashtable();
+      }
+      config.put(key, value);
     }
-    config.put(key, value);
   }
 
   public String getConfig(String key){
@@ -1802,6 +1833,7 @@ break;
   public String getHostKeyAlias(){
     return hostKeyAlias;
   }
+
   public void setServerAliveInterval(int interval) throws JSchException {
     setTimeout(interval);
     this.serverAliveInterval=interval;
@@ -1809,6 +1841,14 @@ break;
   public void setServerAliveCountMax(int count){
     this.serverAliveCountMax=count;
   }
+
+  public int getServerAliveInterval(){
+    return this.serverAliveInterval;
+  }
+  public int getServerAliveCountMax(){
+    return this.serverAliveCountMax;
+  }
+
   public void setDaemonThread(boolean enable){
     this.daemon_thread=enable;
   }
@@ -1825,14 +1865,7 @@ break;
     java.util.Vector result=new java.util.Vector();
     String[] _ciphers=Util.split(ciphers, ",");
     for(int i=0; i<_ciphers.length; i++){
-      try{
-        Class c=Class.forName(getConfig(_ciphers[i]));
-        Cipher _c=(Cipher)(c.newInstance());
-        _c.init(Cipher.ENCRYPT_MODE, 
-                new byte[_c.getBlockSize()],
-                new byte[_c.getIVSize()]);
-      }
-      catch(Exception e){
+      if(!checkCipher(getConfig(_ciphers[i]))){
         result.addElement(_ciphers[i]);
       }
     }
@@ -1850,4 +1883,18 @@ break;
 
     return foo;
   }
+
+  static boolean checkCipher(String cipher){
+    try{
+      Class c=Class.forName(cipher);
+      Cipher _c=(Cipher)(c.newInstance());
+      _c.init(Cipher.ENCRYPT_MODE,
+              new byte[_c.getBlockSize()],
+              new byte[_c.getIVSize()]);
+      return true;
+    }
+    catch(Exception e){
+      return false;
+    }
+  }
 }
