@@ -1,6 +1,6 @@
 /* -*-mode:java; c-basic-offset:2; indent-tabs-mode:nil -*- */
 /*
-Copyright (c) 2002,2003,2004,2005,2006 ymnk, JCraft,Inc. All rights reserved.
+Copyright (c) 2002-2009 ymnk, JCraft,Inc. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
@@ -29,23 +29,25 @@ EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 package com.jcraft.jsch;
 
-abstract class UserAuth{
-  boolean start(Session session) throws Exception{
-    Packet packet=session.packet;
-    Buffer buf=session.buf;
-    // send
-    // byte      SSH_MSG_SERVICE_REQUEST(5)
-    // string    service name "ssh-userauth"
-    packet.reset();
-    buf.putByte((byte)Session.SSH_MSG_SERVICE_REQUEST);
-    buf.putString("ssh-userauth".getBytes());
-    session.write(packet);
+public abstract class UserAuth{
+  protected static final int SSH_MSG_USERAUTH_REQUEST=               50;
+  protected static final int SSH_MSG_USERAUTH_FAILURE=               51;
+  protected static final int SSH_MSG_USERAUTH_SUCCESS=               52;
+  protected static final int SSH_MSG_USERAUTH_BANNER=                53;
+  protected static final int SSH_MSG_USERAUTH_INFO_REQUEST=          60;
+  protected static final int SSH_MSG_USERAUTH_INFO_RESPONSE=         61;
+  protected static final int SSH_MSG_USERAUTH_PK_OK=                 60;
 
-    // receive
-    // byte      SSH_MSG_SERVICE_ACCEPT(6)
-    // string    service name
-    buf=session.read(buf);
-    //System.out.println("read: 6 ? "+buf.buffer[5]);
-    return buf.buffer[5]==6;
+  protected UserInfo userinfo;
+  protected Packet packet;
+  protected Buffer buf;
+  protected String username;
+
+  public boolean start(Session session) throws Exception{
+    this.userinfo=session.getUserInfo();
+    this.packet=session.packet;
+    this.buf=packet.getBuffer();
+    this.username=session.getUserName();
+    return true;
   }
 }
