@@ -1,6 +1,6 @@
 /* -*-mode:java; c-basic-offset:2; indent-tabs-mode:nil -*- */
 /*
-Copyright (c) 2002,2003,2004,2005,2006 ymnk, JCraft,Inc. All rights reserved.
+Copyright (c) 2002-2009 ymnk, JCraft,Inc. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
@@ -35,15 +35,16 @@ import javax.crypto.spec.*;
 
 public class HMACMD5 implements MAC{
   private static final String name="hmac-md5";
-  private static final int bsize=16;
+  private static final int BSIZE=16;
   private Mac mac;
-  public int getBlockSize(){return bsize;};
+  public int getBlockSize(){return BSIZE;};
   public void init(byte[] key) throws Exception{
-    if(key.length>bsize){
-      byte[] tmp=new byte[bsize];
-      System.arraycopy(key, 0, tmp, 0, bsize);	  
+    if(key.length>BSIZE){
+      byte[] tmp=new byte[BSIZE];
+      System.arraycopy(key, 0, tmp, 0, BSIZE);	  
       key=tmp;
     }
+
     SecretKeySpec skey=new SecretKeySpec(key, "HmacMD5");
     mac=Mac.getInstance("HmacMD5");
     mac.init(skey);
@@ -60,9 +61,14 @@ public class HMACMD5 implements MAC{
   public void update(byte foo[], int s, int l){
     mac.update(foo, s, l);      
   }
-  public byte[] doFinal(){
-    return mac.doFinal();
+  public void doFinal(byte[] buf, int offset){
+    try{
+      mac.doFinal(buf, offset);
+    }
+    catch(ShortBufferException e){
+    }
   }
+
   public String getName(){
     return name;
   }
